FUNCTION_BLOCK Lcd_Initialize
VAR CONSTANT

    DEF_CLOCK_SPEED             : I2C_CLK           := I2C_CLK_100K;
    DEF_USE_STRETCH_MODE        : BOOL              := FALSE;
    DEF_I2C_UNIT                : USINT             := 0;

END_VAR

VAR_INPUT

    xEnable_i                   : BOOL;

END_VAR

VAR_OUTPUT

    I2cHandle_o                 : HI2C              := -1;
    usiError_o                  : USINT             := 0;
    xConfirm_o                  : BOOL              := FALSE;

END_VAR

VAR

    FB_I2cInit                  : I2C_INIT;
    FB_Lcd_Write_Cmd            : Lcd_Write_Cmd;

    I2cHandle                   : HI2C;

    bCommand                    : BYTE;
    bMode                       : BYTE;

    xConfirm                    : BOOL;
    usiError                    : USINT;

    FB_RTrig                    : R_TRIG;
    iProcState                  : INT               := 0;

END_VAR


    FB_RTrig (CLK := xEnable_i);
    IF (FB_RTrig.Q = TRUE) THEN
        iProcState := 0;
        usiError_o := 0;
        xConfirm_o := FALSE;
    END_IF;

    IF (xEnable_i = FALSE) THEN
        RETURN;
    END_IF;


    CASE iProcState OF

    (* ------------------------------------------------------------------------------------ *)
    (*  I2C_INIT                                                                            *)
    (* ------------------------------------------------------------------------------------ *)

    0:  FB_I2cInit (ENABLE := FALSE);
        iProcState := iProcState + 1;

    1:
        FB_I2cInit (
            ENABLE           := TRUE,
            CLOCK_SPEED      := DEF_CLOCK_SPEED,
            USE_STRETCH_MODE := DEF_USE_STRETCH_MODE,
            I2C_UNIT         := DEF_I2C_UNIT
            |
            I2cHandle        := I2C_HANDLE,
            xConfirm         := CONFIRM,
            usiError         := ERROR);

        IF (xConfirm = TRUE) THEN
            iProcState := iProcState + 1;
            RETURN;
        END_IF;

        IF (usiError <> 0) THEN
            iProcState := -1;
        END_IF;




    (* ------------------------------------------------------------------------------------ *)
    (*  lcddriver.init.lcd_write(0x03)                                                      *)
    (* ------------------------------------------------------------------------------------ *)

    2:  FB_Lcd_Write_Cmd (xEnable_i := FALSE);
        iProcState := iProcState + 1;

    3:
        bCommand := 16#03;
        bMode    := 16#00;
        FB_Lcd_Write_Cmd (
            xEnable_i  := TRUE,
            bCommand_i := bCommand,
            bMode_i    := bMode
            |
            xConfirm   := xConfirm_o,
            usiError   := usiError_o);

        IF (xConfirm = TRUE) THEN
            iProcState := iProcState + 1;
            RETURN;
        END_IF;

        IF (usiError <> 0) THEN
            iProcState := -1;
        END_IF;


    (* ------------------------------------------------------------------------------------ *)
    (*  lcddriver.init.lcd_write(0x03)                                                      *)
    (* ------------------------------------------------------------------------------------ *)

    4:  FB_Lcd_Write_Cmd (xEnable_i := FALSE);
        iProcState := iProcState + 1;

    5:
        bCommand := 16#03;
        bMode    := 16#00;
        FB_Lcd_Write_Cmd (
            xEnable_i   := TRUE,
            I2cHandle_i := I2cHandle,
            bCommand_i  := bCommand,
            bMode_i     := bMode
            |
            xConfirm    := xConfirm_o,
            usiError    := usiError_o);

        IF (xConfirm = TRUE) THEN
            iProcState := iProcState + 1;
            RETURN;
        END_IF;

        IF (usiError <> 0) THEN
            iProcState := -1;
        END_IF;


    (* ------------------------------------------------------------------------------------ *)
    (*  lcddriver.init.lcd_write(0x03)                                                      *)
    (* ------------------------------------------------------------------------------------ *)

    6:  FB_Lcd_Write_Cmd (xEnable_i := FALSE);
        iProcState := iProcState + 1;

    7:
        bCommand := 16#03;
        bMode    := 16#00;
        FB_Lcd_Write_Cmd (
            xEnable_i   := TRUE,
            I2cHandle_i := I2cHandle,
            bCommand_i  := bCommand,
            bMode_i     := bMode
            |
            xConfirm    := xConfirm_o,
            usiError    := usiError_o);

        IF (xConfirm = TRUE) THEN
            iProcState := iProcState + 1;
            RETURN;
        END_IF;

        IF (usiError <> 0) THEN
            iProcState := -1;
        END_IF;


    (* ------------------------------------------------------------------------------------ *)
    (*  lcddriver.init.lcd_write(0x02)                                                      *)
    (* ------------------------------------------------------------------------------------ *)

    8:  FB_Lcd_Write_Cmd (xEnable_i := FALSE);
        iProcState := iProcState + 1;

    9:
        bCommand := 16#02;
        bMode    := 16#00;
        FB_Lcd_Write_Cmd (
            xEnable_i   := TRUE,
            I2cHandle_i := I2cHandle,
            bCommand_i  := bCommand,
            bMode_i     := bMode
            |
            xConfirm    := xConfirm_o,
            usiError    := usiError_o);

        IF (xConfirm = TRUE) THEN
            iProcState := iProcState + 1;
            RETURN;
        END_IF;

        IF (usiError <> 0) THEN
            iProcState := -1;
        END_IF;


    (* ------------------------------------------------------------------------------------ *)
    (*  lcddriver.init.lcd_write(LCD_FUNCTIONSET | LCD_2LINE | LCD_5x8DOTS | LCD_4BITMODE)  *)
    (* ------------------------------------------------------------------------------------ *)

    10: FB_Lcd_Write_Cmd (xEnable_i := FALSE);
        iProcState := iProcState + 1;

    11:
        bCommand := LCD_FUNCTIONSET OR LCD_2LINE OR LCD_5x8DOTS OR LCD_4BITMODE;
        bMode    := 16#00;
        FB_Lcd_Write_Cmd (
            xEnable_i   := TRUE,
            I2cHandle_i := I2cHandle,
            bCommand_i  := bCommand,
            bMode_i     := bMode
            |
            xConfirm    := xConfirm_o,
            usiError    := usiError_o);

        IF (xConfirm = TRUE) THEN
            iProcState := iProcState + 1;
            RETURN;
        END_IF;

        IF (usiError <> 0) THEN
            iProcState := -1;
        END_IF;


    (* ------------------------------------------------------------------------------------ *)
    (*  lcddriver.init.lcd_write(LCD_DISPLAYCONTROL | LCD_DISPLAYON)                        *)
    (* ------------------------------------------------------------------------------------ *)

    12: FB_Lcd_Write_Cmd (xEnable_i := FALSE);
        iProcState := iProcState + 1;

    13:
        bCommand := LCD_DISPLAYCONTROL OR LCD_DISPLAYON;
        bMode    := 16#00;
        FB_Lcd_Write_Cmd (
            xEnable_i   := TRUE,
            I2cHandle_i := I2cHandle,
            bCommand_i  := bCommand,
            bMode_i     := bMode
            |
            xConfirm    := xConfirm_o,
            usiError    := usiError_o);

        IF (xConfirm = TRUE) THEN
            iProcState := iProcState + 1;
            RETURN;
        END_IF;

        IF (usiError <> 0) THEN
            iProcState := -1;
        END_IF;


    (* ------------------------------------------------------------------------------------ *)
    (*  lcddriver.init.lcd_write(LCD_CLEARDISPLAY)                                          *)
    (* ------------------------------------------------------------------------------------ *)

    14: FB_Lcd_Write_Cmd (xEnable_i := FALSE);
        iProcState := iProcState + 1;

    15:
        bCommand := LCD_CLEARDISPLAY;
        bMode    := 16#00;
        FB_Lcd_Write_Cmd (
            xEnable_i   := TRUE,
            I2cHandle_i := I2cHandle,
            bCommand_i  := bCommand,
            bMode_i     := bMode
            |
            xConfirm    := xConfirm_o,
            usiError    := usiError_o);

        IF (xConfirm = TRUE) THEN
            iProcState := iProcState + 1;
            RETURN;
        END_IF;

        IF (usiError <> 0) THEN
            iProcState := -1;
        END_IF;


    (* ------------------------------------------------------------------------------------ *)
    (*  lcddriver.init.lcd_write(LCD_ENTRYMODESET | LCD_ENTRYLEFT)                          *)
    (* ------------------------------------------------------------------------------------ *)

    16: FB_Lcd_Write_Cmd (xEnable_i := FALSE);
        iProcState := iProcState + 1;

    17:
        bCommand := LCD_ENTRYMODESET OR LCD_ENTRYLEFT;
        bMode    := 16#00;
        FB_Lcd_Write_Cmd (
            xEnable_i   := TRUE,
            I2cHandle_i := I2cHandle,
            bCommand_i  := bCommand,
            bMode_i     := bMode
            |
            xConfirm    := xConfirm_o,
            usiError    := usiError_o);

        IF (xConfirm = TRUE) THEN
            iProcState := iProcState + 1;
            RETURN;
        END_IF;

        IF (usiError <> 0) THEN
            iProcState := -1;
        END_IF;


    (* ------------------------------------------------------------------------------------ *)
    (*  Finish                                                                              *)
    (* ------------------------------------------------------------------------------------ *)

    18: (* ---- Finish ---- *)
        I2cHandle_o := I2cHandle;
        usiError_o  := 0;
        xConfirm_o  := TRUE;
        RETURN;


    (* ------------------------------------------------------------------------------------ *)
    (*  Error                                                                               *)
    (* ------------------------------------------------------------------------------------ *)

    -1:  (* ---- Error ---- *)
        usiError_o := usiError;
        xConfirm_o := FALSE;
        RETURN;

    ELSE
        RETURN;

    END_CASE;



    RETURN;


END_FUNCTION_BLOCK